@@ -51,17 +51,20 @@ public:
 
 private:
     bool fetchIBNR();
+    bool fetchUIC();
     bool fetchGaresConnexions();
     bool fetchCountryInformation();
     QUrl insertOrMerge(const QJsonObject &obj, bool mergeOnly = false);
     void processStations();
     void writeStationData(QIODevice *out);
     void writeIBNRMap(QIODevice *out);
+    void writeUICMap(QIODevice *out);
     void writeGareConnexionMap(QIODevice *out);
     void printSummary();
 
     std::vector<Station> m_stations;
     std::map<uint32_t, QUrl> m_ibnrMap;
+    std::map<uint32_t, QUrl> m_uicMap;
     std::map<QString, QUrl> m_garesConnexionsIdMap;
     Timezones m_tzDb;
 
